#include <config.h>

// #define NEWBUFFERING
// #define DEBUG_RESYNC

/* 1 frame = 4608 byte PCM */

#define LOCAL static

#include        <stdlib.h>
#include        <stdio.h>
#include        <string.h>
#include        <signal.h>
#include        <math.h>

#include "mpg123.h"
#include "huffman.h"
#include "mp3.h"

#include "d_cpu.h"

int MP3_frames;
int MP3_eof;
int MP3_pause;
int MP3_framesize;		// current framesize
int MP3_bitrate;		// current bitrate
int MP3_samplerate;		// current samplerate
int MP3_resync;
int MP3_channels;
int MP3_bps = 2;

static long outscale  = 32768;
#include "tabinit.c"

/*       Frame reader           */

#define MAXFRAMESIZE 1280

static unsigned char bsbuf[MAXFRAMESIZE];

static int bitindex;
static unsigned char *wordpointer;
static int bitsleft;
static int ssize;

static unsigned char pcm_sample[4*1152];
int pcm_point = 0;

static struct frame fr;

static int tabsel_123[2][3][16] = {
   { {0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,},
     {0,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,},
     {0,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,} },

   { {0,32,48,56,64,80,96,112,128,144,160,176,192,224,256,},
     {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,},
     {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,} }
};

static long freqs[9] = {
	44100, 48000, 32000, 22050, 24000, 16000, 11025, 12000, 8000
};

LOCAL unsigned int getbits(short number_of_bits)
{
	unsigned long rval;

	if ((bitsleft -= number_of_bits) < 0)
		return 0;
	if (!number_of_bits)
		return 0;
	rval = wordpointer[0];
	rval <<= 8;
	rval |= wordpointer[1];
	rval <<= 8;
	rval |= wordpointer[2];
	rval <<= bitindex;
	rval &= 0xffffff;
	bitindex += number_of_bits;
	rval >>= (24 - number_of_bits);
	wordpointer += (bitindex >> 3);
	bitindex &= 7;
	return rval;
}


LOCAL unsigned int getbits_fast(short number_of_bits)
{
	unsigned long rval;

	if ((bitsleft -= number_of_bits) < 0)
		return 0;
	if (!number_of_bits)
		return 0;
	rval = wordpointer[0];
	rval <<= 8;
	rval |= wordpointer[1];
	rval <<= bitindex;
	rval &= 0xffff;
	bitindex += number_of_bits;
	rval >>= (16 - number_of_bits);
	wordpointer += (bitindex >> 3);
	bitindex &= 7;
	return rval;
}

LOCAL unsigned int get1bit(void)
{
	unsigned char rval;

	if (--bitsleft < 0)
		return 0;
	rval = (*wordpointer >> (7 - bitindex)) & 1;
	if (++bitindex > 7) {
		++wordpointer;
		bitindex = 0;
	}
	return rval;
}

/*
 * decode a header and write the information
 * into the frame structure
 */
LOCAL int decode_header(struct frame *fr, unsigned long newhead)
{
	// head_check:
	if ((newhead & 0xffe00000) != 0xffe00000)
		return FALSE;
	if (((newhead >> 12) & 0xf) == 0xf)
		return FALSE;
	if (((newhead >> 10) & 0x3) == 0x3 )
		return FALSE;

	fr->lay = 4 - ((newhead >> 17) & 3);

	if (newhead & ((long)1 << 20)) {
		fr->lsf = ~(newhead >> 19) & 1;
		fr->mpeg25 = 0;
	} else {
		fr->lsf = 1;
		fr->mpeg25 = 1;
	}

	if(fr->mpeg25)
		fr->sampling_frequency = 6 + ((newhead >> 10) & 0x3);
	else
		fr->sampling_frequency = ((newhead >> 10) & 0x3) + (fr->lsf * 3);

	fr->error_protection = ~(newhead >> 16) & 0x1;
	fr->bitrate_index = ((newhead >> 12) & 0xf);
	fr->padding   = ((newhead >> 9) & 0x1);
	fr->extension = ((newhead >> 8) & 0x1);
	fr->mode      = ((newhead >> 6) & 0x3);
	fr->mode_ext  = ((newhead >> 4) & 0x3);
	fr->copyright = ((newhead >> 3) & 0x1);
	fr->original  = ((newhead >> 2) & 0x1);
	fr->emphasis  = newhead & 0x3;

	MP3_channels = fr->stereo = (fr->mode == MPG_MD_MONO) ? 1 : 2;

	if(!fr->bitrate_index){
//      	fprintf(stderr,"Free format not supported.\n");
		return FALSE;
	}

	switch (fr->lay) {
	case 2:
		MP3_bitrate = tabsel_123[fr->lsf][1][fr->bitrate_index];
		MP3_samplerate = freqs[fr->sampling_frequency];
		fr->framesize = (long) MP3_bitrate * 144000;
		fr->framesize /= MP3_samplerate;
		MP3_framesize = fr->framesize + fr->padding;
		fr->framesize += fr->padding - 4;
		break;

	case 3:
		if(fr->lsf)
			ssize = (fr->stereo == 1) ? 9 : 17;
		else
			ssize = (fr->stereo == 1) ? 17 : 32;
		if (fr->error_protection)
			ssize += 2;

		MP3_bitrate = tabsel_123[fr->lsf][2][fr->bitrate_index];
		MP3_samplerate = freqs[fr->sampling_frequency];
		fr->framesize  = (long) MP3_bitrate * 144000;
		fr->framesize /= MP3_samplerate << fr->lsf;
		MP3_framesize = fr->framesize;
		fr->framesize += fr->padding - 4;
		break;

	default:
//		fprintf(stderr,"Sorry, unsupported layer type.\n");
		return 0;
	}
	if (fr->framesize <= 0 || fr->framesize > MAXFRAMESIZE)
		return FALSE;

	return 1;
}

#include "layer2.c"
#include "dct64.c"
#include "decode.c"

#if 0
#include "dct36.c"
#include "dct12.c"
#include "layer3.c"
#endif

/******************************************************************************/
/*           PUBLIC FUNCTIONS                  */
/******************************************************************************/

static int tables_done_flag;

static int header_bytes;
static unsigned int header;
static int body_bytes;

// Init decoder tables.  Call first, once!
void MP3_Init()
{
#ifdef ARCH_X86
	_CpuID=CpuDetect();
	_i586=ipentium();
#ifdef HAVE_3DNOW
	_3dnow=a3dnow();
#endif

	printf( "mp3lib: Processor ID: %x\n",_CpuID );
	printf( "mp3lib: i586 processor %sdetected.\n",(_i586?"":"not ") );
#ifdef HAVE_3DNOW
	printf( "mp3lib: AMD 3dnow! extension %sdetected.\n",(a3dnow()?"":"not ") );
#endif
#endif /* ARCH_X86 */

	make_decode_tables(outscale);
	fr.synth = synth_1to1;
	fr.synth_mono = synth_1to1_mono2stereo;
	fr.down_sample = 0;
	fr.down_sample_sblimit = SBLIMIT >> fr.down_sample;
	init_layer2();
#if 0
	init_layer3(fr.down_sample_sblimit);
#endif
	tables_done_flag=1;
	header_bytes = header = 0;
	body_bytes = 0;
}

void MP3_decode_data(unsigned char *data, int nbytes,
		     void (*output)(unsigned char *, int))
{
	unsigned char *end;
	int nb;

	if (nbytes < 0) {
		/* flush */
		header_bytes = body_bytes = 0;
		return;
	}
	end = data + nbytes;
	while (data < end) {
		if (header_bytes < 4) {
			header = (header << 8) + *data++;
			if (++header_bytes == 4)
				if (!decode_header(&fr, header))
					header_bytes = 3;
			continue;
		}
		nb = fr.framesize - body_bytes;
		if (data + nb > end)
			nb = end - data;
		memcpy(bsbuf + body_bytes, data, nb);
		body_bytes += nb;
		data += nb;
		if (body_bytes >= fr.framesize) {
			bitindex = 0;
			wordpointer = bsbuf;
			bitsleft = 8 * body_bytes;
			pcm_point = 0;
			if (fr.error_protection)
				getbits(16);		/* XXX should check CRC */
			switch(fr.lay) {
			case 2:
				do_layer2(&fr, 0);
				break;
#if 0
			case 3: 
				do_layer3(&fr, 0);
				break;
#endif
			}
			if (pcm_point)
				output(pcm_sample, pcm_point);
			body_bytes = header_bytes = 0;
			header = 0;
		}
	}
}

// Prints last frame header in ascii.
void MP3_PrintHeader()
{
        static char *modes[4] = { "Stereo", "Joint-Stereo", "Dual-Channel", "Single-Channel" };
        static char *layers[4] = { "???" , "I", "II", "III" };

        printf("\rMPEG %s, Layer %s, %ld Hz %d kbit %s, BPF : %ld\n",
	       fr.mpeg25 ? "2.5" : (fr.lsf ? "2.0" : "1.0"),
	       layers[fr.lay],freqs[fr.sampling_frequency],
	       tabsel_123[fr.lsf][fr.lay-1][fr.bitrate_index],
	       modes[fr.mode],fr.framesize+4);
        printf("Channels: %d, copyright: %s, original: %s, CRC: %s, emphasis: %d.\n\n",
	       fr.stereo, fr.copyright?"Yes":"No",
	       fr.original?"Yes":"No", fr.error_protection?"Yes":"No",
	       fr.emphasis);
}
